/**********************************************************************
 *
 * Filename:    led.c
 * 
 * Description: LED device driver.
 *
 * Notes:       This file is specific to the Corebell board.
 * 
 * Copyright (c) 2006 Anthony Massa and Michael Barr. All rights reserved.
 * This code is from the book Programming Embedded Systems, With C and
 * GNU Development Tools, 2nd Edition.
 * It is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://www.oreilly.com/catalog/embsys2.
 *
 **********************************************************************/

#include "stdint.h"
#include "pxa270.h"
#include "lds4000.h"
#include "led.h"


#define PIN23_FUNC_GENERAL          (0xFFFF3FFF)
#define GPIO_0_LEVEL_REG_OFFSET     (0x00 >> 2)
#define GPIO_0_DIRECTION_REG_OFFSET (0x0C >> 2)
#define GPIO_0_SET_REG_OFFSET       (0x18 >> 2)
#define GPIO_0_CLEAR_REG_OFFSET     (0x24 >> 2)
#define GPIO_0_FUNC_HI_REG_OFFSET   (0x58 >> 2)


extern uint32_t *gGpioRegisters;


/**********************************************************************
 *
 * Function:    ledInit
 *
 * Description: Initialize the GPIO pin that controls the LED.
 *
 * Notes:       This function is specific to the Arcom board.
 *
 * Returns:     None.
 *
 **********************************************************************/
void ledInit(void)
{
    /* Turn the GPIO pin voltage off, which will light the LED. This should
     * be done before the pins are configured. */
    *(gGpioRegisters + GPIO_0_CLEAR_REG_OFFSET) = LED_GREEN_GPIO23;

    /* Make sure the LED control pin is set to perform general
     * purpose functions. RedBoot may have changed the pin's operation. */
    *(gGpioRegisters + GPIO_0_FUNC_HI_REG_OFFSET) &= PIN23_FUNC_GENERAL;

    /* Set the LED control pin to operate as output. */
    *(gGpioRegisters + GPIO_0_DIRECTION_REG_OFFSET) |= LED_GREEN_GPIO23;
}


/**********************************************************************
 *
 * Function:    ledToggle
 *
 * Description: Toggle the state of one LED.
 *
 * Notes:       This function is specific to the Arcom board.
 *
 * Returns:     None.
 *
 **********************************************************************/
void ledToggle(void)
{
    /* Check the current state of the LED control pin. Then change the
     * state accordingly. */
    if ((*(gGpioRegisters + GPIO_0_LEVEL_REG_OFFSET)) & LED_GREEN_GPIO23)
        *(gGpioRegisters + GPIO_0_CLEAR_REG_OFFSET) = LED_GREEN_GPIO23;
    else
        *(gGpioRegisters + GPIO_0_SET_REG_OFFSET) = LED_GREEN_GPIO23;
}

