/**********************************************************************
 *
 * Filename:    main.c
 * 
 * Description: A simple test program for the memtest.
 *
 * Notes:       
 *
 * Copyright (c) 2006 Anthony Massa and Michael Barr. All rights reserved.
 * This code is from the book Programming Embedded Systems, With C and
 * GNU Development Tools, 2nd Edition.
 * It is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://www.oreilly.com/catalog/embsys2.
 *
 **********************************************************************/

/**********************************************************************
 *
 * Notes:       This file is specific to the LDS4000 board
 * 
 * Copyright (c) 2007 Corebell System, Inc. All rights reserved.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://corebell.co.kr
 *
 **********************************************************************/

#include <stdio.h>
#include <string.h>

#include "stdint.h"
#include "lds4000.h"
#include "memtest.h"
#include "sseg.h"


#define BASE_ADDRESS                (datum *)(0xA0500000)
#define NUM_BYTES                   (0x10000)


/**********************************************************************
 *
 * Function:    main
 *
 * Description: Test a 64 KB block of DRAM.
 * 
 * Notes:       
 *
 * Returns:     0 on failure.
 *              1 on success.
 *
 *
 *            7seg pin map of LDS4000 
 *                              1 : 0000 0001           1 : off
 *             2                2 : 0000 0010           0 : on 
 *           -----              3 : 0000 0100
 *          |6    |1            4 : 0000 1000
 *          |  7  |             5 : 0001 0000
 *           -----              6 : 0010 0000
 *          |5    |3            7 : 0100 0000
 *          |  4  |             8 : 1000 0000
 *           -----  .8
 *
 *      1st seg position : 0x0  upper 8 bit     : base addr : 0x0700_0000
 *      2nd seg position : 0x4  lower 8 bit     : base addr : 0x0700_0004
 *      3rd seg position : 0x8  upper 8 bit     : base addr : 0x0700_0008
 *      4th seg position : 0x12 lower 8 bit     : base addr : 0x0700_000C
 *
 *      ERR.	: 1000 0101 , 1000 1000 , 1000 1000,  0111 1111
 *      	  0x0085,     0x8800,     0x0088,     0x7F00
 *      dOnE	: 1010 0010 , 1100 0000 , 1010 1011,  1000 0101
 *      	  0x00A2,     0xC000,     0x00AB,     0x8500
 *
 **********************************************************************/



int main(void)
{
    datum *pFailAddr;

    /* Configure the 7-segments. */
    SSegInit();

    /* Make sure 7-SEGs are off before we start the memory test. */
   SSEGSet(0xFF, 0xFF00, 0xFF, 0xFF00);

    if ((memtestDataBus(BASE_ADDRESS, &pFailAddr) != 1) ||
        (memtestAddressBus(BASE_ADDRESS, NUM_BYTES, &pFailAddr) != 1) ||
        (memtestDevice(BASE_ADDRESS, NUM_BYTES, &pFailAddr) != 1))
    {
	SSEGSet(0x85, 0x8800, 0x88, 0x7F00); 	/* ERR. */
        return 0;
    }
    else
    {
	SSEGSet(0xA2, 0xC000, 0xAB, 0x8500); 	/* dOnE */
        return 1;
    }
}

