/**********************************************************************
 *
 * Filename:    sseg.c
 * 
 * Description: 7-seg device driver.
 *
 * Notes:       This file is specific to the Corebell board.
 * 
 * Copyright (c) 2006 Anthony Massa and Michael Barr. All rights reserved.
 * It is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://corebell.co.kr
 *
 **********************************************************************/

#include "stdint.h"
#include "pxa270.h"
#include "lds4000.h"
#include "sseg.h"


uint16_t volatile * SEG1, * SEG2, * SEG3, * SEG4; 

/**********************************************************************
 *
 * Function:    SSegInit
 *
 * Description: Initialize the memory maped IO that control the 7-SEGs.
 *
 * Notes:       This function is specific to the Corebell board.
 *
 * Returns:     None.
 *
 * 	
 *
 *            7seg pin map of LDS4000 
 *                              1 : 0000 0001		1 : off
 *             2                2 : 0000 0010		0 : on 
 *           -----              3 : 0000 0100
 *          |6    |1            4 : 0000 1000
 *          |  7  |             5 : 0001 0000
 *           -----              6 : 0010 0000
 *          |5    |3            7 : 0100 0000
 *          |  4  |             8 : 1000 0000
 *           -----  .8
 *
 *	1st seg position : 0x0  upper 8 bit	: base addr : 0x0700_0000
 *	2nd seg position : 0x4  lower 8 bit	: base addr : 0x0700_0004
 *	3rd seg position : 0x8  upper 8 bit	: base addr : 0x0700_0008
 *	4th seg position : 0x12 lower 8 bit	: base addr : 0x0700_000C
 *
 **********************************************************************/

void SSegInit(void)
{

	SEG1 = (uint16_t volatile *) SEG_1_BASE;   
	SEG2 = (uint16_t volatile *) SEG_2_BASE;   
	SEG3 = (uint16_t volatile *) SEG_3_BASE;   
	SEG4 = (uint16_t volatile *) SEG_4_BASE;   


	SSEGSet(0x00FF, 0xFF00, 0x00FF, 0xFF00); 
}

/**********************************************************************
 *
 * Function:    SSEGSet
 *
 * Description: Set up the specified 7-SEGs.
 *
 * Notes:       This function is specific to the Corebell board.
 *
 * Returns:     None.
 *
 **********************************************************************/
void SSEGSet(uint16_t sseg1, uint16_t sseg2, uint16_t sseg3, uint16_t sseg4)
{

	*SEG1 = sseg1; 

	*SEG2 = sseg2; 

	*SEG3 = sseg3; 

	*SEG4 = sseg4; 
}
