/**********************************************************************
 *
 * Filename:    lds4000.h
 * 
 * Description: Header file for Corebell Lds4000 board.
 *
 * Notes:       The constants in this file are specific to the Corebell
 *              board.
 *
 * 
 * Copyright (c) 2006 Anthony Massa and Michael Barr. All rights reserved.
 * This code is from the book Programming Embedded Systems, With C and
 * GNU Development Tools, 2nd Edition.
 * It is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://www.oreilly.com/catalog/embsys2.
 *
 **********************************************************************/

#ifndef _LDS4000_H
#define _LDS4000_H

#ifndef TRUE
#define TRUE            1
#endif

#ifndef FALSE
#define FALSE           0
#endif

#ifndef NULL
#define NULL            (void *)0
#endif


/**********************************************************************
 *
 *  Memory Map
 *
 *  NOTE:   RedBoot remaps memory via the MMU. This explains why this map differs
 *          from the memory map in the LDS4000 technical manual.
 *
 *          Base Address   Size  Description
 *          -------------- ----- -----------------------------------
 *            0x00000000     128M   SDRAM
 *            0x08000300     N/A   Ethernet controller 1
 *            0x14000300     N/A   Ethernet controller 2
 *            0x16000300     N/A   Ethernet controller 3
 *            0x07000000     N/A   7SEGs,LEDs,CharacterLCD,Keypad
 *            			   7SEG : 0x07000000
 *            			   LEDs : 0x07000010
 *            			   CLCD : 0x07000020(control), 0x07000024(data)
 *            			   Key	: 0x07000030
 *            0xF0000000     64M   Flash
 *
 **********************************************************************/

#define SDRAM_BASE                      (0x00000000)
#define ETHERNET_CTRL_BASE              (0x08000300)
#define FLASH_BASE                      (0xF0000000)


/**********************************************************************
 *  Interrupt Map
 **********************************************************************/

#define ETH1_CTRL_INT                    (12)
#define ETH2_CTRL_INT                    (10)
#define ETH3_CTRL_INT                    (81)
#define SERIAL2_INT                      (21)
#define SERIAL1_INT                      (22)
#define TIMER0_INT                       (26)
#define TIMER1_INT                       (27)
#define TIMER2_INT                       (28)
#define TIMER3_INT                       (29)


/**********************************************************************
 *  GPIO Pin Controls for LED of 23th GPIO
 **********************************************************************/

#define LED_GREEN_GPIO23    		(0x00800000)
#define LED_GREEN_GPIO24    		(0x01000000)
#define LED_GREEN_GPIO25    		(0x02000000)


/**********************************************************************
 *  Memory mapped IO 7-SEGs
 **********************************************************************/

#define SEG_1_BASE			(0x07000000)
#define SEG_2_BASE			(0x07000004)
#define SEG_3_BASE			(0x07000008)
#define SEG_4_BASE			(0x0700000C)


/**********************************************************************
 *  Memory mapped IO LEDs
 **********************************************************************/
#define LED_BASE			(0x07000010)


/**********************************************************************
 *  Memory mapped IO CLCD
 **********************************************************************/
#define CLCD_CTL_BASE			(0x07000020)
#define CLCD_DATA_BASE			(0x07000024)


/**********************************************************************
 *  Memory mapped IO Keypad
 **********************************************************************/
#define KEYPAD_BASE			(0x07000030)


#endif /* _LDS4000_H */

